/* 
 * Figshare API
 *
 * Figshare apiv2. Using Swagger 2.0
 *
 * OpenAPI spec version: 2.0.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// ArticleEmbargoUpdater
    /// </summary>
    [DataContract]
    public partial class ArticleEmbargoUpdater :  IEquatable<ArticleEmbargoUpdater>, IValidatableObject
    {
        /// <summary>
        /// Embargo can be enabled at the article or the file level. Possible values: article, file
        /// </summary>
        /// <value>Embargo can be enabled at the article or the file level. Possible values: article, file</value>
        [JsonConverter(typeof(StringEnumConverter))]
        public enum EmbargoTypeEnum
        {
            
            /// <summary>
            /// Enum Article for value: article
            /// </summary>
            [EnumMember(Value = "article")]
            Article = 1,
            
            /// <summary>
            /// Enum File for value: file
            /// </summary>
            [EnumMember(Value = "file")]
            File = 2
        }

        /// <summary>
        /// Embargo can be enabled at the article or the file level. Possible values: article, file
        /// </summary>
        /// <value>Embargo can be enabled at the article or the file level. Possible values: article, file</value>
        [DataMember(Name="embargo_type", EmitDefaultValue=false)]
        public EmbargoTypeEnum EmbargoType { get; set; }
        /// <summary>
        /// Initializes a new instance of the <see cref="ArticleEmbargoUpdater" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected ArticleEmbargoUpdater() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="ArticleEmbargoUpdater" /> class.
        /// </summary>
        /// <param name="isEmbargoed">Embargo status (required).</param>
        /// <param name="embargoDate">Date when the embargo expires and the article gets published, &#39;0&#39; value will set up permanent embargo (required).</param>
        /// <param name="embargoType">Embargo can be enabled at the article or the file level. Possible values: article, file (required).</param>
        /// <param name="embargoTitle">Title for embargo.</param>
        /// <param name="embargoReason">Reason for setting embargo.</param>
        /// <param name="embargoOptions">List of embargo permissions to be associated with the article. The list must contain &#x60;id&#x60; and can also contain &#x60;group_ids&#x60;(a field that only applies to &#39;logged_in&#39; permissions). The new list replaces old options in the database, and an empty list removes all permissions for this article. Administration permission has to be set up alone but logged in and IP range permissions can be set up together..</param>
        public ArticleEmbargoUpdater(bool? isEmbargoed = default(bool?), string embargoDate = default(string), EmbargoTypeEnum embargoType = default(EmbargoTypeEnum), string embargoTitle = default(string), string embargoReason = default(string), List<Object> embargoOptions = default(List<Object>))
        {
            // to ensure "isEmbargoed" is required (not null)
            if (isEmbargoed == null)
            {
                throw new InvalidDataException("isEmbargoed is a required property for ArticleEmbargoUpdater and cannot be null");
            }
            else
            {
                this.IsEmbargoed = isEmbargoed;
            }
            // to ensure "embargoDate" is required (not null)
            if (embargoDate == null)
            {
                throw new InvalidDataException("embargoDate is a required property for ArticleEmbargoUpdater and cannot be null");
            }
            else
            {
                this.EmbargoDate = embargoDate;
            }
            // to ensure "embargoType" is required (not null)
            if (embargoType == null)
            {
                throw new InvalidDataException("embargoType is a required property for ArticleEmbargoUpdater and cannot be null");
            }
            else
            {
                this.EmbargoType = embargoType;
            }
            this.EmbargoTitle = embargoTitle;
            this.EmbargoReason = embargoReason;
            this.EmbargoOptions = embargoOptions;
        }
        
        /// <summary>
        /// Embargo status
        /// </summary>
        /// <value>Embargo status</value>
        [DataMember(Name="is_embargoed", EmitDefaultValue=false)]
        public bool? IsEmbargoed { get; set; }

        /// <summary>
        /// Date when the embargo expires and the article gets published, &#39;0&#39; value will set up permanent embargo
        /// </summary>
        /// <value>Date when the embargo expires and the article gets published, &#39;0&#39; value will set up permanent embargo</value>
        [DataMember(Name="embargo_date", EmitDefaultValue=false)]
        public string EmbargoDate { get; set; }


        /// <summary>
        /// Title for embargo
        /// </summary>
        /// <value>Title for embargo</value>
        [DataMember(Name="embargo_title", EmitDefaultValue=false)]
        public string EmbargoTitle { get; set; }

        /// <summary>
        /// Reason for setting embargo
        /// </summary>
        /// <value>Reason for setting embargo</value>
        [DataMember(Name="embargo_reason", EmitDefaultValue=false)]
        public string EmbargoReason { get; set; }

        /// <summary>
        /// List of embargo permissions to be associated with the article. The list must contain &#x60;id&#x60; and can also contain &#x60;group_ids&#x60;(a field that only applies to &#39;logged_in&#39; permissions). The new list replaces old options in the database, and an empty list removes all permissions for this article. Administration permission has to be set up alone but logged in and IP range permissions can be set up together.
        /// </summary>
        /// <value>List of embargo permissions to be associated with the article. The list must contain &#x60;id&#x60; and can also contain &#x60;group_ids&#x60;(a field that only applies to &#39;logged_in&#39; permissions). The new list replaces old options in the database, and an empty list removes all permissions for this article. Administration permission has to be set up alone but logged in and IP range permissions can be set up together.</value>
        [DataMember(Name="embargo_options", EmitDefaultValue=false)]
        public List<Object> EmbargoOptions { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class ArticleEmbargoUpdater {\n");
            sb.Append("  IsEmbargoed: ").Append(IsEmbargoed).Append("\n");
            sb.Append("  EmbargoDate: ").Append(EmbargoDate).Append("\n");
            sb.Append("  EmbargoType: ").Append(EmbargoType).Append("\n");
            sb.Append("  EmbargoTitle: ").Append(EmbargoTitle).Append("\n");
            sb.Append("  EmbargoReason: ").Append(EmbargoReason).Append("\n");
            sb.Append("  EmbargoOptions: ").Append(EmbargoOptions).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as ArticleEmbargoUpdater);
        }

        /// <summary>
        /// Returns true if ArticleEmbargoUpdater instances are equal
        /// </summary>
        /// <param name="input">Instance of ArticleEmbargoUpdater to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(ArticleEmbargoUpdater input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.IsEmbargoed == input.IsEmbargoed ||
                    (this.IsEmbargoed != null &&
                    this.IsEmbargoed.Equals(input.IsEmbargoed))
                ) && 
                (
                    this.EmbargoDate == input.EmbargoDate ||
                    (this.EmbargoDate != null &&
                    this.EmbargoDate.Equals(input.EmbargoDate))
                ) && 
                (
                    this.EmbargoType == input.EmbargoType ||
                    (this.EmbargoType != null &&
                    this.EmbargoType.Equals(input.EmbargoType))
                ) && 
                (
                    this.EmbargoTitle == input.EmbargoTitle ||
                    (this.EmbargoTitle != null &&
                    this.EmbargoTitle.Equals(input.EmbargoTitle))
                ) && 
                (
                    this.EmbargoReason == input.EmbargoReason ||
                    (this.EmbargoReason != null &&
                    this.EmbargoReason.Equals(input.EmbargoReason))
                ) && 
                (
                    this.EmbargoOptions == input.EmbargoOptions ||
                    this.EmbargoOptions != null &&
                    this.EmbargoOptions.SequenceEqual(input.EmbargoOptions)
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.IsEmbargoed != null)
                    hashCode = hashCode * 59 + this.IsEmbargoed.GetHashCode();
                if (this.EmbargoDate != null)
                    hashCode = hashCode * 59 + this.EmbargoDate.GetHashCode();
                if (this.EmbargoType != null)
                    hashCode = hashCode * 59 + this.EmbargoType.GetHashCode();
                if (this.EmbargoTitle != null)
                    hashCode = hashCode * 59 + this.EmbargoTitle.GetHashCode();
                if (this.EmbargoReason != null)
                    hashCode = hashCode * 59 + this.EmbargoReason.GetHashCode();
                if (this.EmbargoOptions != null)
                    hashCode = hashCode * 59 + this.EmbargoOptions.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
