/* 
 * Figshare API
 *
 * Figshare apiv2. Using Swagger 2.0
 *
 * OpenAPI spec version: 2.0.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// ArticleUpdate
    /// </summary>
    [DataContract]
    public partial class ArticleUpdate :  IEquatable<ArticleUpdate>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ArticleUpdate" /> class.
        /// </summary>
        /// <param name="title">Title of article.</param>
        /// <param name="description">The article description. In a publisher case, usually this is the remote article description (default to &quot;&quot;).</param>
        /// <param name="isMetadataRecord">True if article has no files.</param>
        /// <param name="metadataReason">Article metadata reason.</param>
        /// <param name="tags">List of tags to be associated with the article. Keywords can be used instead.</param>
        /// <param name="keywords">List of tags to be associated with the article. Tags can be used instead.</param>
        /// <param name="references">List of links to be associated with the article (e.g [\&quot;http://link1\&quot;, \&quot;http://link2\&quot;, \&quot;http://link3\&quot;]).</param>
        /// <param name="relatedMaterials">List of related materials; supersedes references and resource DOI/title..</param>
        /// <param name="categories">List of category ids to be associated with the article(e.g [1, 23, 33, 66]).</param>
        /// <param name="categoriesBySourceId">List of category source ids to be associated with the article, supersedes the categories property.</param>
        /// <param name="authors">List of authors to be associated with the article. The list can contain the following fields: id, name, first_name, last_name, email, orcid_id. If an id is supplied, it will take priority and everything else will be ignored. For adding more authors use the specific authors endpoint..</param>
        /// <param name="customFields">List of key, values pairs to be associated with the article.</param>
        /// <param name="customFieldsList">List of custom fields values, supersedes custom_fields parameter.</param>
        /// <param name="definedType">&lt;b&gt;One of:&lt;/b&gt; &lt;code&gt;figure&lt;/code&gt; &lt;code&gt;online resource&lt;/code&gt; &lt;code&gt;preprint&lt;/code&gt; &lt;code&gt;book&lt;/code&gt; &lt;code&gt;conference contribution&lt;/code&gt; &lt;code&gt;media&lt;/code&gt; &lt;code&gt;dataset&lt;/code&gt; &lt;code&gt;poster&lt;/code&gt; &lt;code&gt;journal contribution&lt;/code&gt; &lt;code&gt;presentation&lt;/code&gt; &lt;code&gt;thesis&lt;/code&gt; &lt;code&gt;software&lt;/code&gt;.</param>
        /// <param name="funding">Grant number or funding authority (default to &quot;&quot;).</param>
        /// <param name="fundingList">Funding creation / update items.</param>
        /// <param name="license">License id for this article. (default to 0).</param>
        /// <param name="doi">Not applicable for regular users. In an institutional case, make sure your group supports setting DOIs. This setting is applied by figshare via opening a ticket through our support/helpdesk system. (default to &quot;&quot;).</param>
        /// <param name="handle">Not applicable for regular users. In an institutional case, make sure your group supports setting Handles. This setting is applied by figshare via opening a ticket through our support/helpdesk system. (default to &quot;&quot;).</param>
        /// <param name="resourceDoi">Deprecated by related materials. Not applicable to regular users. In a publisher case, this is the publisher article DOI. (default to &quot;&quot;).</param>
        /// <param name="resourceTitle">Deprecated by related materials. Not applicable to regular users. In a publisher case, this is the publisher article title. (default to &quot;&quot;).</param>
        /// <param name="timeline">Various timeline dates.</param>
        /// <param name="downloadDisabled">If true, downloading of files for this article is disabled.</param>
        /// <param name="groupId">Not applicable to regular users. This field is reserved to institutions/publishers with access to assign to specific groups.</param>
        public ArticleUpdate(string title = default(string), string description = "", bool? isMetadataRecord = default(bool?), string metadataReason = default(string), List<string> tags = default(List<string>), List<string> keywords = default(List<string>), List<string> references = default(List<string>), List<RelatedMaterial> relatedMaterials = default(List<RelatedMaterial>), List<long?> categories = default(List<long?>), List<string> categoriesBySourceId = default(List<string>), List<Object> authors = default(List<Object>), Object customFields = default(Object), List<CustomArticleFieldAdd> customFieldsList = default(List<CustomArticleFieldAdd>), string definedType = default(string), string funding = "", List<FundingCreate> fundingList = default(List<FundingCreate>), long? license = 0, string doi = "", string handle = "", string resourceDoi = "", string resourceTitle = "", TimelineUpdate timeline = default(TimelineUpdate), bool? downloadDisabled = default(bool?), long? groupId = default(long?))
        {
            this.Title = title;
            // use default value if no "description" provided
            if (description == null)
            {
                this.Description = "";
            }
            else
            {
                this.Description = description;
            }
            this.IsMetadataRecord = isMetadataRecord;
            this.MetadataReason = metadataReason;
            this.Tags = tags;
            this.Keywords = keywords;
            this.References = references;
            this.RelatedMaterials = relatedMaterials;
            this.Categories = categories;
            this.CategoriesBySourceId = categoriesBySourceId;
            this.Authors = authors;
            this.CustomFields = customFields;
            this.CustomFieldsList = customFieldsList;
            this.DefinedType = definedType;
            // use default value if no "funding" provided
            if (funding == null)
            {
                this.Funding = "";
            }
            else
            {
                this.Funding = funding;
            }
            this.FundingList = fundingList;
            // use default value if no "license" provided
            if (license == null)
            {
                this.License = 0;
            }
            else
            {
                this.License = license;
            }
            // use default value if no "doi" provided
            if (doi == null)
            {
                this.Doi = "";
            }
            else
            {
                this.Doi = doi;
            }
            // use default value if no "handle" provided
            if (handle == null)
            {
                this.Handle = "";
            }
            else
            {
                this.Handle = handle;
            }
            // use default value if no "resourceDoi" provided
            if (resourceDoi == null)
            {
                this.ResourceDoi = "";
            }
            else
            {
                this.ResourceDoi = resourceDoi;
            }
            // use default value if no "resourceTitle" provided
            if (resourceTitle == null)
            {
                this.ResourceTitle = "";
            }
            else
            {
                this.ResourceTitle = resourceTitle;
            }
            this.Timeline = timeline;
            this.DownloadDisabled = downloadDisabled;
            this.GroupId = groupId;
        }
        
        /// <summary>
        /// Title of article
        /// </summary>
        /// <value>Title of article</value>
        [DataMember(Name="title", EmitDefaultValue=false)]
        public string Title { get; set; }

        /// <summary>
        /// The article description. In a publisher case, usually this is the remote article description
        /// </summary>
        /// <value>The article description. In a publisher case, usually this is the remote article description</value>
        [DataMember(Name="description", EmitDefaultValue=false)]
        public string Description { get; set; }

        /// <summary>
        /// True if article has no files
        /// </summary>
        /// <value>True if article has no files</value>
        [DataMember(Name="is_metadata_record", EmitDefaultValue=false)]
        public bool? IsMetadataRecord { get; set; }

        /// <summary>
        /// Article metadata reason
        /// </summary>
        /// <value>Article metadata reason</value>
        [DataMember(Name="metadata_reason", EmitDefaultValue=false)]
        public string MetadataReason { get; set; }

        /// <summary>
        /// List of tags to be associated with the article. Keywords can be used instead
        /// </summary>
        /// <value>List of tags to be associated with the article. Keywords can be used instead</value>
        [DataMember(Name="tags", EmitDefaultValue=false)]
        public List<string> Tags { get; set; }

        /// <summary>
        /// List of tags to be associated with the article. Tags can be used instead
        /// </summary>
        /// <value>List of tags to be associated with the article. Tags can be used instead</value>
        [DataMember(Name="keywords", EmitDefaultValue=false)]
        public List<string> Keywords { get; set; }

        /// <summary>
        /// List of links to be associated with the article (e.g [\&quot;http://link1\&quot;, \&quot;http://link2\&quot;, \&quot;http://link3\&quot;])
        /// </summary>
        /// <value>List of links to be associated with the article (e.g [\&quot;http://link1\&quot;, \&quot;http://link2\&quot;, \&quot;http://link3\&quot;])</value>
        [DataMember(Name="references", EmitDefaultValue=false)]
        public List<string> References { get; set; }

        /// <summary>
        /// List of related materials; supersedes references and resource DOI/title.
        /// </summary>
        /// <value>List of related materials; supersedes references and resource DOI/title.</value>
        [DataMember(Name="related_materials", EmitDefaultValue=false)]
        public List<RelatedMaterial> RelatedMaterials { get; set; }

        /// <summary>
        /// List of category ids to be associated with the article(e.g [1, 23, 33, 66])
        /// </summary>
        /// <value>List of category ids to be associated with the article(e.g [1, 23, 33, 66])</value>
        [DataMember(Name="categories", EmitDefaultValue=false)]
        public List<long?> Categories { get; set; }

        /// <summary>
        /// List of category source ids to be associated with the article, supersedes the categories property
        /// </summary>
        /// <value>List of category source ids to be associated with the article, supersedes the categories property</value>
        [DataMember(Name="categories_by_source_id", EmitDefaultValue=false)]
        public List<string> CategoriesBySourceId { get; set; }

        /// <summary>
        /// List of authors to be associated with the article. The list can contain the following fields: id, name, first_name, last_name, email, orcid_id. If an id is supplied, it will take priority and everything else will be ignored. For adding more authors use the specific authors endpoint.
        /// </summary>
        /// <value>List of authors to be associated with the article. The list can contain the following fields: id, name, first_name, last_name, email, orcid_id. If an id is supplied, it will take priority and everything else will be ignored. For adding more authors use the specific authors endpoint.</value>
        [DataMember(Name="authors", EmitDefaultValue=false)]
        public List<Object> Authors { get; set; }

        /// <summary>
        /// List of key, values pairs to be associated with the article
        /// </summary>
        /// <value>List of key, values pairs to be associated with the article</value>
        [DataMember(Name="custom_fields", EmitDefaultValue=false)]
        public Object CustomFields { get; set; }

        /// <summary>
        /// List of custom fields values, supersedes custom_fields parameter
        /// </summary>
        /// <value>List of custom fields values, supersedes custom_fields parameter</value>
        [DataMember(Name="custom_fields_list", EmitDefaultValue=false)]
        public List<CustomArticleFieldAdd> CustomFieldsList { get; set; }

        /// <summary>
        /// &lt;b&gt;One of:&lt;/b&gt; &lt;code&gt;figure&lt;/code&gt; &lt;code&gt;online resource&lt;/code&gt; &lt;code&gt;preprint&lt;/code&gt; &lt;code&gt;book&lt;/code&gt; &lt;code&gt;conference contribution&lt;/code&gt; &lt;code&gt;media&lt;/code&gt; &lt;code&gt;dataset&lt;/code&gt; &lt;code&gt;poster&lt;/code&gt; &lt;code&gt;journal contribution&lt;/code&gt; &lt;code&gt;presentation&lt;/code&gt; &lt;code&gt;thesis&lt;/code&gt; &lt;code&gt;software&lt;/code&gt;
        /// </summary>
        /// <value>&lt;b&gt;One of:&lt;/b&gt; &lt;code&gt;figure&lt;/code&gt; &lt;code&gt;online resource&lt;/code&gt; &lt;code&gt;preprint&lt;/code&gt; &lt;code&gt;book&lt;/code&gt; &lt;code&gt;conference contribution&lt;/code&gt; &lt;code&gt;media&lt;/code&gt; &lt;code&gt;dataset&lt;/code&gt; &lt;code&gt;poster&lt;/code&gt; &lt;code&gt;journal contribution&lt;/code&gt; &lt;code&gt;presentation&lt;/code&gt; &lt;code&gt;thesis&lt;/code&gt; &lt;code&gt;software&lt;/code&gt;</value>
        [DataMember(Name="defined_type", EmitDefaultValue=false)]
        public string DefinedType { get; set; }

        /// <summary>
        /// Grant number or funding authority
        /// </summary>
        /// <value>Grant number or funding authority</value>
        [DataMember(Name="funding", EmitDefaultValue=false)]
        public string Funding { get; set; }

        /// <summary>
        /// Funding creation / update items
        /// </summary>
        /// <value>Funding creation / update items</value>
        [DataMember(Name="funding_list", EmitDefaultValue=false)]
        public List<FundingCreate> FundingList { get; set; }

        /// <summary>
        /// License id for this article.
        /// </summary>
        /// <value>License id for this article.</value>
        [DataMember(Name="license", EmitDefaultValue=false)]
        public long? License { get; set; }

        /// <summary>
        /// Not applicable for regular users. In an institutional case, make sure your group supports setting DOIs. This setting is applied by figshare via opening a ticket through our support/helpdesk system.
        /// </summary>
        /// <value>Not applicable for regular users. In an institutional case, make sure your group supports setting DOIs. This setting is applied by figshare via opening a ticket through our support/helpdesk system.</value>
        [DataMember(Name="doi", EmitDefaultValue=false)]
        public string Doi { get; set; }

        /// <summary>
        /// Not applicable for regular users. In an institutional case, make sure your group supports setting Handles. This setting is applied by figshare via opening a ticket through our support/helpdesk system.
        /// </summary>
        /// <value>Not applicable for regular users. In an institutional case, make sure your group supports setting Handles. This setting is applied by figshare via opening a ticket through our support/helpdesk system.</value>
        [DataMember(Name="handle", EmitDefaultValue=false)]
        public string Handle { get; set; }

        /// <summary>
        /// Deprecated by related materials. Not applicable to regular users. In a publisher case, this is the publisher article DOI.
        /// </summary>
        /// <value>Deprecated by related materials. Not applicable to regular users. In a publisher case, this is the publisher article DOI.</value>
        [DataMember(Name="resource_doi", EmitDefaultValue=false)]
        public string ResourceDoi { get; set; }

        /// <summary>
        /// Deprecated by related materials. Not applicable to regular users. In a publisher case, this is the publisher article title.
        /// </summary>
        /// <value>Deprecated by related materials. Not applicable to regular users. In a publisher case, this is the publisher article title.</value>
        [DataMember(Name="resource_title", EmitDefaultValue=false)]
        public string ResourceTitle { get; set; }

        /// <summary>
        /// Various timeline dates
        /// </summary>
        /// <value>Various timeline dates</value>
        [DataMember(Name="timeline", EmitDefaultValue=false)]
        public TimelineUpdate Timeline { get; set; }

        /// <summary>
        /// If true, downloading of files for this article is disabled
        /// </summary>
        /// <value>If true, downloading of files for this article is disabled</value>
        [DataMember(Name="download_disabled", EmitDefaultValue=false)]
        public bool? DownloadDisabled { get; set; }

        /// <summary>
        /// Not applicable to regular users. This field is reserved to institutions/publishers with access to assign to specific groups
        /// </summary>
        /// <value>Not applicable to regular users. This field is reserved to institutions/publishers with access to assign to specific groups</value>
        [DataMember(Name="group_id", EmitDefaultValue=false)]
        public long? GroupId { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class ArticleUpdate {\n");
            sb.Append("  Title: ").Append(Title).Append("\n");
            sb.Append("  Description: ").Append(Description).Append("\n");
            sb.Append("  IsMetadataRecord: ").Append(IsMetadataRecord).Append("\n");
            sb.Append("  MetadataReason: ").Append(MetadataReason).Append("\n");
            sb.Append("  Tags: ").Append(Tags).Append("\n");
            sb.Append("  Keywords: ").Append(Keywords).Append("\n");
            sb.Append("  References: ").Append(References).Append("\n");
            sb.Append("  RelatedMaterials: ").Append(RelatedMaterials).Append("\n");
            sb.Append("  Categories: ").Append(Categories).Append("\n");
            sb.Append("  CategoriesBySourceId: ").Append(CategoriesBySourceId).Append("\n");
            sb.Append("  Authors: ").Append(Authors).Append("\n");
            sb.Append("  CustomFields: ").Append(CustomFields).Append("\n");
            sb.Append("  CustomFieldsList: ").Append(CustomFieldsList).Append("\n");
            sb.Append("  DefinedType: ").Append(DefinedType).Append("\n");
            sb.Append("  Funding: ").Append(Funding).Append("\n");
            sb.Append("  FundingList: ").Append(FundingList).Append("\n");
            sb.Append("  License: ").Append(License).Append("\n");
            sb.Append("  Doi: ").Append(Doi).Append("\n");
            sb.Append("  Handle: ").Append(Handle).Append("\n");
            sb.Append("  ResourceDoi: ").Append(ResourceDoi).Append("\n");
            sb.Append("  ResourceTitle: ").Append(ResourceTitle).Append("\n");
            sb.Append("  Timeline: ").Append(Timeline).Append("\n");
            sb.Append("  DownloadDisabled: ").Append(DownloadDisabled).Append("\n");
            sb.Append("  GroupId: ").Append(GroupId).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as ArticleUpdate);
        }

        /// <summary>
        /// Returns true if ArticleUpdate instances are equal
        /// </summary>
        /// <param name="input">Instance of ArticleUpdate to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(ArticleUpdate input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Title == input.Title ||
                    (this.Title != null &&
                    this.Title.Equals(input.Title))
                ) && 
                (
                    this.Description == input.Description ||
                    (this.Description != null &&
                    this.Description.Equals(input.Description))
                ) && 
                (
                    this.IsMetadataRecord == input.IsMetadataRecord ||
                    (this.IsMetadataRecord != null &&
                    this.IsMetadataRecord.Equals(input.IsMetadataRecord))
                ) && 
                (
                    this.MetadataReason == input.MetadataReason ||
                    (this.MetadataReason != null &&
                    this.MetadataReason.Equals(input.MetadataReason))
                ) && 
                (
                    this.Tags == input.Tags ||
                    this.Tags != null &&
                    this.Tags.SequenceEqual(input.Tags)
                ) && 
                (
                    this.Keywords == input.Keywords ||
                    this.Keywords != null &&
                    this.Keywords.SequenceEqual(input.Keywords)
                ) && 
                (
                    this.References == input.References ||
                    this.References != null &&
                    this.References.SequenceEqual(input.References)
                ) && 
                (
                    this.RelatedMaterials == input.RelatedMaterials ||
                    this.RelatedMaterials != null &&
                    this.RelatedMaterials.SequenceEqual(input.RelatedMaterials)
                ) && 
                (
                    this.Categories == input.Categories ||
                    this.Categories != null &&
                    this.Categories.SequenceEqual(input.Categories)
                ) && 
                (
                    this.CategoriesBySourceId == input.CategoriesBySourceId ||
                    this.CategoriesBySourceId != null &&
                    this.CategoriesBySourceId.SequenceEqual(input.CategoriesBySourceId)
                ) && 
                (
                    this.Authors == input.Authors ||
                    this.Authors != null &&
                    this.Authors.SequenceEqual(input.Authors)
                ) && 
                (
                    this.CustomFields == input.CustomFields ||
                    (this.CustomFields != null &&
                    this.CustomFields.Equals(input.CustomFields))
                ) && 
                (
                    this.CustomFieldsList == input.CustomFieldsList ||
                    this.CustomFieldsList != null &&
                    this.CustomFieldsList.SequenceEqual(input.CustomFieldsList)
                ) && 
                (
                    this.DefinedType == input.DefinedType ||
                    (this.DefinedType != null &&
                    this.DefinedType.Equals(input.DefinedType))
                ) && 
                (
                    this.Funding == input.Funding ||
                    (this.Funding != null &&
                    this.Funding.Equals(input.Funding))
                ) && 
                (
                    this.FundingList == input.FundingList ||
                    this.FundingList != null &&
                    this.FundingList.SequenceEqual(input.FundingList)
                ) && 
                (
                    this.License == input.License ||
                    (this.License != null &&
                    this.License.Equals(input.License))
                ) && 
                (
                    this.Doi == input.Doi ||
                    (this.Doi != null &&
                    this.Doi.Equals(input.Doi))
                ) && 
                (
                    this.Handle == input.Handle ||
                    (this.Handle != null &&
                    this.Handle.Equals(input.Handle))
                ) && 
                (
                    this.ResourceDoi == input.ResourceDoi ||
                    (this.ResourceDoi != null &&
                    this.ResourceDoi.Equals(input.ResourceDoi))
                ) && 
                (
                    this.ResourceTitle == input.ResourceTitle ||
                    (this.ResourceTitle != null &&
                    this.ResourceTitle.Equals(input.ResourceTitle))
                ) && 
                (
                    this.Timeline == input.Timeline ||
                    (this.Timeline != null &&
                    this.Timeline.Equals(input.Timeline))
                ) && 
                (
                    this.DownloadDisabled == input.DownloadDisabled ||
                    (this.DownloadDisabled != null &&
                    this.DownloadDisabled.Equals(input.DownloadDisabled))
                ) && 
                (
                    this.GroupId == input.GroupId ||
                    (this.GroupId != null &&
                    this.GroupId.Equals(input.GroupId))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Title != null)
                    hashCode = hashCode * 59 + this.Title.GetHashCode();
                if (this.Description != null)
                    hashCode = hashCode * 59 + this.Description.GetHashCode();
                if (this.IsMetadataRecord != null)
                    hashCode = hashCode * 59 + this.IsMetadataRecord.GetHashCode();
                if (this.MetadataReason != null)
                    hashCode = hashCode * 59 + this.MetadataReason.GetHashCode();
                if (this.Tags != null)
                    hashCode = hashCode * 59 + this.Tags.GetHashCode();
                if (this.Keywords != null)
                    hashCode = hashCode * 59 + this.Keywords.GetHashCode();
                if (this.References != null)
                    hashCode = hashCode * 59 + this.References.GetHashCode();
                if (this.RelatedMaterials != null)
                    hashCode = hashCode * 59 + this.RelatedMaterials.GetHashCode();
                if (this.Categories != null)
                    hashCode = hashCode * 59 + this.Categories.GetHashCode();
                if (this.CategoriesBySourceId != null)
                    hashCode = hashCode * 59 + this.CategoriesBySourceId.GetHashCode();
                if (this.Authors != null)
                    hashCode = hashCode * 59 + this.Authors.GetHashCode();
                if (this.CustomFields != null)
                    hashCode = hashCode * 59 + this.CustomFields.GetHashCode();
                if (this.CustomFieldsList != null)
                    hashCode = hashCode * 59 + this.CustomFieldsList.GetHashCode();
                if (this.DefinedType != null)
                    hashCode = hashCode * 59 + this.DefinedType.GetHashCode();
                if (this.Funding != null)
                    hashCode = hashCode * 59 + this.Funding.GetHashCode();
                if (this.FundingList != null)
                    hashCode = hashCode * 59 + this.FundingList.GetHashCode();
                if (this.License != null)
                    hashCode = hashCode * 59 + this.License.GetHashCode();
                if (this.Doi != null)
                    hashCode = hashCode * 59 + this.Doi.GetHashCode();
                if (this.Handle != null)
                    hashCode = hashCode * 59 + this.Handle.GetHashCode();
                if (this.ResourceDoi != null)
                    hashCode = hashCode * 59 + this.ResourceDoi.GetHashCode();
                if (this.ResourceTitle != null)
                    hashCode = hashCode * 59 + this.ResourceTitle.GetHashCode();
                if (this.Timeline != null)
                    hashCode = hashCode * 59 + this.Timeline.GetHashCode();
                if (this.DownloadDisabled != null)
                    hashCode = hashCode * 59 + this.DownloadDisabled.GetHashCode();
                if (this.GroupId != null)
                    hashCode = hashCode * 59 + this.GroupId.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // Title (string) maxLength
            if(this.Title != null && this.Title.Length > 1000)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Title, length must be less than 1000.", new [] { "Title" });
            }

            // Title (string) minLength
            if(this.Title != null && this.Title.Length < 3)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Title, length must be greater than 3.", new [] { "Title" });
            }

            // Description (string) maxLength
            if(this.Description != null && this.Description.Length > 10000)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Description, length must be less than 10000.", new [] { "Description" });
            }

            yield break;
        }
    }

}
