/* 
 * Figshare API
 *
 * Figshare apiv2. Using Swagger 2.0
 *
 * OpenAPI spec version: 2.0.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// FileCreator
    /// </summary>
    [DataContract]
    public partial class FileCreator :  IEquatable<FileCreator>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="FileCreator" /> class.
        /// </summary>
        /// <param name="link">Url for an existing file that will not be uploaded to Figshare.</param>
        /// <param name="md5">MD5 sum pre-computed on client side..</param>
        /// <param name="name">File name including the extension; can be omitted only for linked files..</param>
        /// <param name="size">File size in bytes; can be omitted only for linked files..</param>
        /// <param name="folderPath">Unix-style directory path of the file; only available if the file was uploaded within a folder structure.</param>
        public FileCreator(string link = default(string), string md5 = default(string), string name = default(string), long? size = default(long?), string folderPath = default(string))
        {
            this.Link = link;
            this.Md5 = md5;
            this.Name = name;
            this.Size = size;
            this.FolderPath = folderPath;
        }
        
        /// <summary>
        /// Url for an existing file that will not be uploaded to Figshare
        /// </summary>
        /// <value>Url for an existing file that will not be uploaded to Figshare</value>
        [DataMember(Name="link", EmitDefaultValue=false)]
        public string Link { get; set; }

        /// <summary>
        /// MD5 sum pre-computed on client side.
        /// </summary>
        /// <value>MD5 sum pre-computed on client side.</value>
        [DataMember(Name="md5", EmitDefaultValue=false)]
        public string Md5 { get; set; }

        /// <summary>
        /// File name including the extension; can be omitted only for linked files.
        /// </summary>
        /// <value>File name including the extension; can be omitted only for linked files.</value>
        [DataMember(Name="name", EmitDefaultValue=false)]
        public string Name { get; set; }

        /// <summary>
        /// File size in bytes; can be omitted only for linked files.
        /// </summary>
        /// <value>File size in bytes; can be omitted only for linked files.</value>
        [DataMember(Name="size", EmitDefaultValue=false)]
        public long? Size { get; set; }

        /// <summary>
        /// Unix-style directory path of the file; only available if the file was uploaded within a folder structure
        /// </summary>
        /// <value>Unix-style directory path of the file; only available if the file was uploaded within a folder structure</value>
        [DataMember(Name="folder_path", EmitDefaultValue=false)]
        public string FolderPath { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class FileCreator {\n");
            sb.Append("  Link: ").Append(Link).Append("\n");
            sb.Append("  Md5: ").Append(Md5).Append("\n");
            sb.Append("  Name: ").Append(Name).Append("\n");
            sb.Append("  Size: ").Append(Size).Append("\n");
            sb.Append("  FolderPath: ").Append(FolderPath).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as FileCreator);
        }

        /// <summary>
        /// Returns true if FileCreator instances are equal
        /// </summary>
        /// <param name="input">Instance of FileCreator to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(FileCreator input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Link == input.Link ||
                    (this.Link != null &&
                    this.Link.Equals(input.Link))
                ) && 
                (
                    this.Md5 == input.Md5 ||
                    (this.Md5 != null &&
                    this.Md5.Equals(input.Md5))
                ) && 
                (
                    this.Name == input.Name ||
                    (this.Name != null &&
                    this.Name.Equals(input.Name))
                ) && 
                (
                    this.Size == input.Size ||
                    (this.Size != null &&
                    this.Size.Equals(input.Size))
                ) && 
                (
                    this.FolderPath == input.FolderPath ||
                    (this.FolderPath != null &&
                    this.FolderPath.Equals(input.FolderPath))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Link != null)
                    hashCode = hashCode * 59 + this.Link.GetHashCode();
                if (this.Md5 != null)
                    hashCode = hashCode * 59 + this.Md5.GetHashCode();
                if (this.Name != null)
                    hashCode = hashCode * 59 + this.Name.GetHashCode();
                if (this.Size != null)
                    hashCode = hashCode * 59 + this.Size.GetHashCode();
                if (this.FolderPath != null)
                    hashCode = hashCode * 59 + this.FolderPath.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }

}
