/* 
 * Figshare API
 *
 * Figshare apiv2. Using Swagger 2.0
 *
 * OpenAPI spec version: 2.0.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */

using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// ProjectCreate
    /// </summary>
    [DataContract]
    public partial class ProjectCreate :  IEquatable<ProjectCreate>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ProjectCreate" /> class.
        /// </summary>
        [JsonConstructorAttribute]
        protected ProjectCreate() { }
        /// <summary>
        /// Initializes a new instance of the <see cref="ProjectCreate" /> class.
        /// </summary>
        /// <param name="title">The title for this project - mandatory. 3 - 1000 characters. (required).</param>
        /// <param name="description">Project description.</param>
        /// <param name="funding">Grant number or organization(s) that funded this project. Up to 2000 characters permitted..</param>
        /// <param name="fundingList">Funding creation / update items.</param>
        /// <param name="groupId">Only if project type is group..</param>
        /// <param name="customFields">List of key, values pairs to be associated with the project.</param>
        /// <param name="customFieldsList">List of custom fields values, supersedes custom_fields parameter.</param>
        public ProjectCreate(string title = default(string), string description = default(string), string funding = default(string), List<FundingCreate> fundingList = default(List<FundingCreate>), long? groupId = default(long?), Object customFields = default(Object), List<CustomArticleFieldAdd> customFieldsList = default(List<CustomArticleFieldAdd>))
        {
            // to ensure "title" is required (not null)
            if (title == null)
            {
                throw new InvalidDataException("title is a required property for ProjectCreate and cannot be null");
            }
            else
            {
                this.Title = title;
            }
            this.Description = description;
            this.Funding = funding;
            this.FundingList = fundingList;
            this.GroupId = groupId;
            this.CustomFields = customFields;
            this.CustomFieldsList = customFieldsList;
        }
        
        /// <summary>
        /// The title for this project - mandatory. 3 - 1000 characters.
        /// </summary>
        /// <value>The title for this project - mandatory. 3 - 1000 characters.</value>
        [DataMember(Name="title", EmitDefaultValue=false)]
        public string Title { get; set; }

        /// <summary>
        /// Project description
        /// </summary>
        /// <value>Project description</value>
        [DataMember(Name="description", EmitDefaultValue=false)]
        public string Description { get; set; }

        /// <summary>
        /// Grant number or organization(s) that funded this project. Up to 2000 characters permitted.
        /// </summary>
        /// <value>Grant number or organization(s) that funded this project. Up to 2000 characters permitted.</value>
        [DataMember(Name="funding", EmitDefaultValue=false)]
        public string Funding { get; set; }

        /// <summary>
        /// Funding creation / update items
        /// </summary>
        /// <value>Funding creation / update items</value>
        [DataMember(Name="funding_list", EmitDefaultValue=false)]
        public List<FundingCreate> FundingList { get; set; }

        /// <summary>
        /// Only if project type is group.
        /// </summary>
        /// <value>Only if project type is group.</value>
        [DataMember(Name="group_id", EmitDefaultValue=false)]
        public long? GroupId { get; set; }

        /// <summary>
        /// List of key, values pairs to be associated with the project
        /// </summary>
        /// <value>List of key, values pairs to be associated with the project</value>
        [DataMember(Name="custom_fields", EmitDefaultValue=false)]
        public Object CustomFields { get; set; }

        /// <summary>
        /// List of custom fields values, supersedes custom_fields parameter
        /// </summary>
        /// <value>List of custom fields values, supersedes custom_fields parameter</value>
        [DataMember(Name="custom_fields_list", EmitDefaultValue=false)]
        public List<CustomArticleFieldAdd> CustomFieldsList { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class ProjectCreate {\n");
            sb.Append("  Title: ").Append(Title).Append("\n");
            sb.Append("  Description: ").Append(Description).Append("\n");
            sb.Append("  Funding: ").Append(Funding).Append("\n");
            sb.Append("  FundingList: ").Append(FundingList).Append("\n");
            sb.Append("  GroupId: ").Append(GroupId).Append("\n");
            sb.Append("  CustomFields: ").Append(CustomFields).Append("\n");
            sb.Append("  CustomFieldsList: ").Append(CustomFieldsList).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as ProjectCreate);
        }

        /// <summary>
        /// Returns true if ProjectCreate instances are equal
        /// </summary>
        /// <param name="input">Instance of ProjectCreate to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(ProjectCreate input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Title == input.Title ||
                    (this.Title != null &&
                    this.Title.Equals(input.Title))
                ) && 
                (
                    this.Description == input.Description ||
                    (this.Description != null &&
                    this.Description.Equals(input.Description))
                ) && 
                (
                    this.Funding == input.Funding ||
                    (this.Funding != null &&
                    this.Funding.Equals(input.Funding))
                ) && 
                (
                    this.FundingList == input.FundingList ||
                    this.FundingList != null &&
                    this.FundingList.SequenceEqual(input.FundingList)
                ) && 
                (
                    this.GroupId == input.GroupId ||
                    (this.GroupId != null &&
                    this.GroupId.Equals(input.GroupId))
                ) && 
                (
                    this.CustomFields == input.CustomFields ||
                    (this.CustomFields != null &&
                    this.CustomFields.Equals(input.CustomFields))
                ) && 
                (
                    this.CustomFieldsList == input.CustomFieldsList ||
                    this.CustomFieldsList != null &&
                    this.CustomFieldsList.SequenceEqual(input.CustomFieldsList)
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Title != null)
                    hashCode = hashCode * 59 + this.Title.GetHashCode();
                if (this.Description != null)
                    hashCode = hashCode * 59 + this.Description.GetHashCode();
                if (this.Funding != null)
                    hashCode = hashCode * 59 + this.Funding.GetHashCode();
                if (this.FundingList != null)
                    hashCode = hashCode * 59 + this.FundingList.GetHashCode();
                if (this.GroupId != null)
                    hashCode = hashCode * 59 + this.GroupId.GetHashCode();
                if (this.CustomFields != null)
                    hashCode = hashCode * 59 + this.CustomFields.GetHashCode();
                if (this.CustomFieldsList != null)
                    hashCode = hashCode * 59 + this.CustomFieldsList.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            // Title (string) maxLength
            if(this.Title != null && this.Title.Length > 1000)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Title, length must be less than 1000.", new [] { "Title" });
            }

            // Title (string) minLength
            if(this.Title != null && this.Title.Length < 3)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Title, length must be greater than 3.", new [] { "Title" });
            }

            // Description (string) maxLength
            if(this.Description != null && this.Description.Length > 10000)
            {
                yield return new System.ComponentModel.DataAnnotations.ValidationResult("Invalid value for Description, length must be less than 10000.", new [] { "Description" });
            }

            yield break;
        }
    }

}
